# This file was auto-generated by Fern from our API Definition.

import typing

TraceExperimentItemBulkWriteViewSource = typing.Union[
    typing.Literal["sdk", "experiment", "playground", "optimization"], typing.Any
]
