# This file was auto-generated by Fern from our API Definition.

import typing

RetentionRulePublicRetention = typing.Union[
    typing.Literal["short_14d", "base_60d", "extended_400d", "unlimited"], typing.Any
]
