# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .local_runner_job_result_request_status import LocalRunnerJobResultRequestStatus

__all__ = ["LocalRunnerJobResultRequestStatus"]
