# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .agent_blueprint_write import AgentBlueprintWrite


class AgentConfigCreateWrite(UniversalBaseModel):
    project_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Project ID. Either project_id or project_name must be provided
    """

    project_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Project name. Either project_id or project_name must be provided
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Agent config ID. Generated automatically if not provided
    """

    blueprint: AgentBlueprintWrite

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
