# This file was auto-generated by Fern from our API Definition.

import typing

AgentConfigValueHistoryType = typing.Union[
    typing.Literal["string", "integer", "float", "boolean", "prompt", "prompt_commit"], typing.Any
]
