# This file was auto-generated by Fern from our API Definition.

import typing

LocalRunnerJobStatus = typing.Union[
    typing.Literal["pending", "running", "completed", "failed", "cancelled"], typing.Any
]
