# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .project_metric_response_public_interval import ProjectMetricResponsePublicInterval
from .project_metric_response_public_metric_type import ProjectMetricResponsePublicMetricType
from .results_number_public import ResultsNumberPublic


class ProjectMetricResponsePublic(UniversalBaseModel):
    project_id: typing.Optional[str] = None
    metric_type: typing.Optional[ProjectMetricResponsePublicMetricType] = None
    interval: typing.Optional[ProjectMetricResponsePublicInterval] = None
    results: typing.Optional[typing.List[ResultsNumberPublic]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
