# This file was auto-generated by Fern from our API Definition.

import typing

ProjectMetricResponsePublicMetricType = typing.Union[
    typing.Literal[
        "FEEDBACK_SCORES",
        "TRACE_COUNT",
        "TOKEN_USAGE",
        "DURATION",
        "COST",
        "GUARDRAILS_FAILED_COUNT",
        "THREAD_COUNT",
        "THREAD_DURATION",
        "THREAD_FEEDBACK_SCORES",
        "SPAN_FEEDBACK_SCORES",
        "SPAN_COUNT",
        "SPAN_DURATION",
        "SPAN_TOKEN_USAGE",
    ],
    typing.Any,
]
